# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("msacm32.dll")
prototypes = \
    {
        # 
        'acmGetVersion': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        # 
        'acmMetrics': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hao", "uMetric", "pMetric"]),
        # 
        'acmDriverEnum': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmDriverID': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hao", "phadid", "fdwDriverID"]),
        # 
        'acmDriverAddA': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phadid", "hinstModule", "lParam", "dwPriority", "fdwAdd"]),
        # 
        'acmDriverAddW': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phadid", "hinstModule", "lParam", "dwPriority", "fdwAdd"]),
        # 
        'acmDriverRemove': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hadid", "fdwRemove"]),
        # 
        'acmDriverOpen': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phad", "hadid", "fdwOpen"]),
        # 
        'acmDriverClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "fdwClose"]),
        # 
        'acmDriverMessage': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["had", "uMsg", "lParam1", "lParam2"]),
        # 
        'acmDriverPriority': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hadid", "dwPriority", "fdwPriority"]),
        # 
        'acmDriverDetailsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fccType": SimTypeInt(signed=False, label="UInt32"), "fccComp": SimTypeInt(signed=False, label="UInt32"), "wMid": SimTypeShort(signed=False, label="UInt16"), "wPid": SimTypeShort(signed=False, label="UInt16"), "vdwACM": SimTypeInt(signed=False, label="UInt32"), "vdwDriver": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cFormatTags": SimTypeInt(signed=False, label="UInt32"), "cFilterTags": SimTypeInt(signed=False, label="UInt32"), "hicon": SimTypeBottom(label="HICON"), "szShortName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 32), "szLongName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128), "szCopyright": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 80), "szLicensing": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128), "szFeatures": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 512)}, name="ACMDRIVERDETAILSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hadid", "padd", "fdwDetails"]),
        # 
        'acmDriverDetailsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fccType": SimTypeInt(signed=False, label="UInt32"), "fccComp": SimTypeInt(signed=False, label="UInt32"), "wMid": SimTypeShort(signed=False, label="UInt16"), "wPid": SimTypeShort(signed=False, label="UInt16"), "vdwACM": SimTypeInt(signed=False, label="UInt32"), "vdwDriver": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cFormatTags": SimTypeInt(signed=False, label="UInt32"), "cFilterTags": SimTypeInt(signed=False, label="UInt32"), "hicon": SimTypeBottom(label="HICON"), "szShortName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 32), "szLongName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "szCopyright": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 80), "szLicensing": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "szFeatures": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 512)}, name="ACMDRIVERDETAILSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hadid", "padd", "fdwDetails"]),
        # 
        'acmFormatTagDetailsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFORMATTAGDETAILSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fdwDetails"]),
        # 
        'acmFormatTagDetailsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFORMATTAGDETAILSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fdwDetails"]),
        # 
        'acmFormatTagEnumA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFORMATTAGDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFORMATTAGDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFormatTagEnumW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFORMATTAGDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "cbFormatSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFormats": SimTypeInt(signed=False, label="UInt32"), "szFormatTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFORMATTAGDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFormatDetailsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFORMATDETAILSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fdwDetails"]),
        # 
        'acmFormatDetailsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="tACMFORMATDETAILSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fdwDetails"]),
        # 
        'acmFormatEnumA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFORMATDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFORMATDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFormatEnumW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="tACMFORMATDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFormatIndex": SimTypeInt(signed=False, label="UInt32"), "dwFormatTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "szFormat": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="tACMFORMATDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFormatSuggest': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pwfxSrc", "pwfxDst", "cbwfxDst", "fdwSuggest"]),
        # 
        'acmFormatChooseA': SimTypeFunction([SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStyle": SimTypeInt(signed=False, label="UInt32"), "hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "pszTitle": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "szFormatTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48), "szFormat": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128), "pszName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cchName": SimTypeInt(signed=False, label="UInt32"), "fdwEnum": SimTypeInt(signed=False, label="UInt32"), "pwfxEnum": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "hInstance": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszTemplateName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "lCustData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnHook": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)}, name="ACMFORMATCHOOSEA", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pafmtc"]),
        # 
        'acmFormatChooseW': SimTypeFunction([SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStyle": SimTypeInt(signed=False, label="UInt32"), "hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pwfx": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "cbwfx": SimTypeInt(signed=False, label="UInt32"), "pszTitle": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szFormatTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48), "szFormat": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "pszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cchName": SimTypeInt(signed=False, label="UInt32"), "fdwEnum": SimTypeInt(signed=False, label="UInt32"), "pwfxEnum": SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), "hInstance": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszTemplateName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lCustData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnHook": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)}, name="ACMFORMATCHOOSEW", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pafmtc"]),
        # 
        'acmFilterTagDetailsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFILTERTAGDETAILSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fdwDetails"]),
        # 
        'acmFilterTagDetailsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFILTERTAGDETAILSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fdwDetails"]),
        # 
        'acmFilterTagEnumA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFILTERTAGDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48)}, name="ACMFILTERTAGDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFilterTagEnumW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFILTERTAGDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTagIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "cbFilterSize": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "cStandardFilters": SimTypeInt(signed=False, label="UInt32"), "szFilterTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48)}, name="ACMFILTERTAGDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "paftd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "paftd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFilterDetailsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFILTERDETAILSA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fdwDetails"]),
        # 
        'acmFilterDetailsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="ACMFILTERDETAILSW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fdwDetails"]),
        # 
        'acmFilterEnumA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFILTERDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128)}, name="ACMFILTERDETAILSA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFilterEnumW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="ACMFILTERDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterIndex": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwSupport": SimTypeInt(signed=False, label="UInt32"), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "szFilter": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128)}, name="ACMFILTERDETAILSW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hadid", "pafd", "dwInstance", "fdwSupport"]), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["had", "pafd", "fnCallback", "dwInstance", "fdwEnum"]),
        # 
        'acmFilterChooseA': SimTypeFunction([SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStyle": SimTypeInt(signed=False, label="UInt32"), "hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "pszTitle": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "szFilterTag": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 48), "szFilter": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 128), "pszName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cchName": SimTypeInt(signed=False, label="UInt32"), "fdwEnum": SimTypeInt(signed=False, label="UInt32"), "pwfltrEnum": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "hInstance": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszTemplateName": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "lCustData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnHook": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)}, name="ACMFILTERCHOOSEA", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pafltrc"]),
        # 
        'acmFilterChooseW': SimTypeFunction([SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStyle": SimTypeInt(signed=False, label="UInt32"), "hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pwfltr": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "cbwfltr": SimTypeInt(signed=False, label="UInt32"), "pszTitle": SimTypePointer(SimTypeChar(label="Char"), offset=0), "szFilterTag": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 48), "szFilter": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "pszName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "cchName": SimTypeInt(signed=False, label="UInt32"), "fdwEnum": SimTypeInt(signed=False, label="UInt32"), "pwfltrEnum": SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), "hInstance": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pszTemplateName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lCustData": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "pfnHook": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hwnd", "uMsg", "wParam", "lParam"]), offset=0)}, name="ACMFILTERCHOOSEW", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pafltrc"]),
        # 
        'acmStreamOpen': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"wFormatTag": SimTypeShort(signed=False, label="UInt16"), "nChannels": SimTypeShort(signed=False, label="UInt16"), "nSamplesPerSec": SimTypeInt(signed=False, label="UInt32"), "nAvgBytesPerSec": SimTypeInt(signed=False, label="UInt32"), "nBlockAlign": SimTypeShort(signed=False, label="UInt16"), "wBitsPerSample": SimTypeShort(signed=False, label="UInt16"), "cbSize": SimTypeShort(signed=False, label="UInt16")}, name="WAVEFORMATEX", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "dwFilterTag": SimTypeInt(signed=False, label="UInt32"), "fdwFilter": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 5)}, name="WAVEFILTER", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["phas", "had", "pwfxSrc", "pwfxDst", "pwfltr", "dwCallback", "dwInstance", "fdwOpen"]),
        # 
        'acmStreamClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "fdwClose"]),
        # 
        'acmStreamSize': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "cbInput", "pdwOutputBytes", "fdwSize"]),
        # 
        'acmStreamReset': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "fdwReset"]),
        # 
        'acmStreamMessage': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "uMsg", "lParam1", "lParam2"]),
        # 
        'acmStreamConvert': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStatus": SimTypeInt(signed=False, label="UInt32"), "dwUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbSrc": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbSrcLength": SimTypeInt(signed=False, label="UInt32"), "cbSrcLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwSrcUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbDst": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbDstLength": SimTypeInt(signed=False, label="UInt32"), "cbDstLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwDstUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwReservedDriver": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 10)}, name="ACMSTREAMHEADER", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "pash", "fdwConvert"]),
        # 
        'acmStreamPrepareHeader': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStatus": SimTypeInt(signed=False, label="UInt32"), "dwUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbSrc": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbSrcLength": SimTypeInt(signed=False, label="UInt32"), "cbSrcLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwSrcUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbDst": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbDstLength": SimTypeInt(signed=False, label="UInt32"), "cbDstLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwDstUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwReservedDriver": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 10)}, name="ACMSTREAMHEADER", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "pash", "fdwPrepare"]),
        # 
        'acmStreamUnprepareHeader': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbStruct": SimTypeInt(signed=False, label="UInt32"), "fdwStatus": SimTypeInt(signed=False, label="UInt32"), "dwUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbSrc": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbSrcLength": SimTypeInt(signed=False, label="UInt32"), "cbSrcLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwSrcUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "pbDst": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "cbDstLength": SimTypeInt(signed=False, label="UInt32"), "cbDstLengthUsed": SimTypeInt(signed=False, label="UInt32"), "dwDstUser": SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), "dwReservedDriver": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 10)}, name="ACMSTREAMHEADER", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["has", "pash", "fdwUnprepare"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("xmllite.dll")
prototypes = \
    {
        # 
        'CreateXmlReader': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeBottom(label="IMalloc")], SimTypeInt(signed=True, label="Int32"), arg_names=["riid", "ppvObject", "pMalloc"]),
        # 
        'CreateXmlReaderInputWithEncodingCodePage': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IMalloc"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pInputStream", "pMalloc", "nEncodingCodePage", "fEncodingHint", "pwszBaseUri", "ppInput"]),
        # 
        'CreateXmlReaderInputWithEncodingName': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IMalloc"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pInputStream", "pMalloc", "pwszEncodingName", "fEncodingHint", "pwszBaseUri", "ppInput"]),
        # 
        'CreateXmlWriter': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0), SimTypeBottom(label="IMalloc")], SimTypeInt(signed=True, label="Int32"), arg_names=["riid", "ppvObject", "pMalloc"]),
        # 
        'CreateXmlWriterOutputWithEncodingCodePage': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IMalloc"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pOutputStream", "pMalloc", "nEncodingCodePage", "ppOutput"]),
        # 
        'CreateXmlWriterOutputWithEncodingName': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IMalloc"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pOutputStream", "pMalloc", "pwszEncodingName", "ppOutput"]),
    }

lib.set_prototypes(prototypes)



import angr

######################################
# __libc_init
#
# Refer to http://androidxref.com/5.1.1_r6/xref/bionic/libc/bionic/libc_init_dynamic.cpp
# and http://androidxref.com/5.1.1_r6/xref/bionic/libc/private/KernelArgumentBlock.h
# raw_args points to argc, *argv, and *envp located on the stack
# unused is always zero
# slingshot points to main()
# structors points to PRE_INIT_ARRAY, INIT_ARRAY, and FINI_ARRAY
######################################
class __libc_init(angr.SimProcedure):
    #pylint:disable=arguments-differ,unused-argument,attribute-defined-outside-init

    ADDS_EXITS = True
    NO_RET = True
    local_vars = ('main', 'argc', 'argv', 'envp')

    def run(self, raw_args, unused, slingshot, structors):
        offset = self.state.arch.bytes
        readlen = self.state.arch.bytes
        endness = self.state.arch.memory_endness
        self.main = slingshot
        self.argc = self.state.memory.load(raw_args + 0 * offset, readlen, endness=endness)
        argc_val = self.state.solver.eval(self.argc)
        self.argv = self.state.memory.load(raw_args + 1 * offset, readlen, endness=endness)
        self.envp= self.state.memory.load(raw_args + (1 + argc_val + 1) * offset, readlen, endness=endness)
        # TODO: __cxa_atexit calls for various at-exit needs
        self.call(self.main, (self.argc, self.argv, self.envp), 'after_slingshot',
            prototype='int main(int arch, char **argv, char **envp)')

    def after_slingshot(self, raw_args, unused, slingshot, structors, exit_addr=0):
        self.exit(0)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("dsound.dll")
prototypes = \
    {
        # 
        'DirectSoundCreate': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSound"), offset=0), SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcGuidDevice", "ppDS", "pUnkOuter"]),
        # 
        'DirectSoundEnumerateA': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDSEnumCallback", "pContext"]),
        # 
        'DirectSoundEnumerateW': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDSEnumCallback", "pContext"]),
        # 
        'DirectSoundCaptureCreate': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSoundCapture"), offset=0), SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcGuidDevice", "ppDSC", "pUnkOuter"]),
        # 
        'DirectSoundCaptureEnumerateA': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDSEnumCallback", "pContext"]),
        # 
        'DirectSoundCaptureEnumerateW': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["param0", "param1", "param2", "param3"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pDSEnumCallback", "pContext"]),
        # 
        'DirectSoundCreate8': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSound8"), offset=0), SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcGuidDevice", "ppDS8", "pUnkOuter"]),
        # 
        'DirectSoundCaptureCreate8': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSoundCapture"), offset=0), SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcGuidDevice", "ppDSC8", "pUnkOuter"]),
        # 
        'DirectSoundFullDuplexCreate': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwBufferBytes": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeInt(signed=False, label="UInt32"), "lpwfxFormat": SimTypePointer(SimTypeBottom(label="WAVEFORMATEX"), offset=0), "dwFXCount": SimTypeInt(signed=False, label="UInt32"), "lpDSCFXDesc": SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "guidDSCFXClass": SimTypeBottom(label="Guid"), "guidDSCFXInstance": SimTypeBottom(label="Guid"), "dwReserved1": SimTypeInt(signed=False, label="UInt32"), "dwReserved2": SimTypeInt(signed=False, label="UInt32")}, name="DSCEFFECTDESC", pack=False, align=None), offset=0)}, name="DSCBUFFERDESC", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "dwBufferBytes": SimTypeInt(signed=False, label="UInt32"), "dwReserved": SimTypeInt(signed=False, label="UInt32"), "lpwfxFormat": SimTypePointer(SimTypeBottom(label="WAVEFORMATEX"), offset=0), "guid3DAlgorithm": SimTypeBottom(label="Guid")}, name="DSBUFFERDESC", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IDirectSoundFullDuplex"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSoundCaptureBuffer8"), offset=0), SimTypePointer(SimTypeBottom(label="IDirectSoundBuffer8"), offset=0), SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pcGuidCaptureDevice", "pcGuidRenderDevice", "pcDSCBufferDesc", "pcDSBufferDesc", "hWnd", "dwLevel", "ppDSFD", "ppDSCBuffer8", "ppDSBuffer8", "pUnkOuter"]),
        # 
        'GetDeviceID': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pGuidSrc", "pGuidDest"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("mf.dll")
prototypes = \
    {
        # 
        'MFCreateMediaSession': SimTypeFunction([SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFMediaSession"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pConfiguration", "ppMediaSession"]),
        # 
        'MFCreatePMPMediaSession': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFMediaSession"), offset=0), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCreationFlags", "pConfiguration", "ppMediaSession", "ppEnablerActivate"]),
        # 
        'MFCreateTopology': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFTopology"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppTopo"]),
        # 
        'MFCreateTopologyNode': SimTypeFunction([SimTypeInt(signed=False, label="MF_TOPOLOGY_TYPE"), SimTypePointer(SimTypeBottom(label="IMFTopologyNode"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["NodeType", "ppNode"]),
        # 
        'MFGetTopoNodeCurrentType': SimTypeFunction([SimTypeBottom(label="IMFTopologyNode"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="IMFMediaType"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pNode", "dwStreamIndex", "fOutput", "ppType"]),
        # 
        'MFGetService': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["punkObject", "guidService", "riid", "ppvObject"]),
        # 
        'MFCreatePresentationClock': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFPresentationClock"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppPresentationClock"]),
        # 
        'MFRequireProtectedEnvironment': SimTypeFunction([SimTypeBottom(label="IMFPresentationDescriptor")], SimTypeInt(signed=True, label="Int32"), arg_names=["pPresentationDescriptor"]),
        # 
        'MFCreateSimpleTypeHandler': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFMediaTypeHandler"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppHandler"]),
        # 
        'MFShutdownObject': SimTypeFunction([SimTypeBottom(label="IUnknown")], SimTypeInt(signed=True, label="Int32"), arg_names=["pUnk"]),
        # 
        'MFCreateAudioRenderer': SimTypeFunction([SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAudioAttributes", "ppSink"]),
        # 
        'MFCreateAudioRendererActivate': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppActivate"]),
        # 
        'MFCreateVideoRendererActivate': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hwndVideo", "ppActivate"]),
        # 
        'MFCreateMPEG4MediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IMFMediaType"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIByteStream", "pVideoMediaType", "pAudioMediaType", "ppIMediaSink"]),
        # 
        'MFCreate3GPMediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IMFMediaType"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIByteStream", "pVideoMediaType", "pAudioMediaType", "ppIMediaSink"]),
        # 
        'MFCreateMP3MediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTargetByteStream", "ppMediaSink"]),
        # 
        'MFCreateAC3MediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFMediaType"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTargetByteStream", "pAudioMediaType", "ppMediaSink"]),
        # 
        'MFCreateADTSMediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFMediaType"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pTargetByteStream", "pAudioMediaType", "ppMediaSink"]),
        # 
        'MFCreateMuxSink': SimTypeFunction([SimTypeBottom(label="Guid"), SimTypeBottom(label="IMFAttributes"), SimTypeBottom(label="IMFByteStream"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["guidOutputSubType", "pOutputAttributes", "pOutputByteStream", "ppMuxSink"]),
        # 
        'MFCreateFMPEG4MediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IMFMediaType"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIByteStream", "pVideoMediaType", "pAudioMediaType", "ppIMediaSink"]),
        # 
        'MFCreateTopoLoader': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFTopoLoader"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppObj"]),
        # 
        'MFCreateSampleGrabberSinkActivate': SimTypeFunction([SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IMFSampleGrabberSinkCallback"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIMFMediaType", "pIMFSampleGrabberSinkCallback", "ppIActivate"]),
        # 
        'MFCreateStandardQualityManager': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFQualityManager"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppQualityManager"]),
        # 
        'MFCreateSequencerSource': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="IMFSequencerSource"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pReserved", "ppSequencerSource"]),
        # 
        'MFCreateSequencerSegmentOffset': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeLongLong(signed=True, label="Int64"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"cVal": SimTypeBottom(label="CHAR"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "lVal": SimTypeInt(signed=True, label="Int32"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "hVal": SimTypeBottom(label="LARGE_INTEGER"), "uhVal": SimTypeBottom(label="ULARGE_INTEGER"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "filetime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "puuid": SimTypePointer(SimTypeBottom(label="Guid"), offset=0), "pclipdata": SimTypePointer(SimTypeBottom(label="CLIPDATA"), offset=0), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "bstrblobVal": SimTypeBottom(label="BSTRBLOB"), "blob": SimTypeBottom(label="BLOB"), "pszVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pwszVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "pStream": SimTypeBottom(label="IStream"), "pStorage": SimTypeBottom(label="IStorage"), "pVersionedStream": SimTypePointer(SimStruct({"guidVersion": SimTypeBottom(label="Guid"), "pStream": SimTypeBottom(label="IStream")}, name="VERSIONEDSTREAM", pack=False, align=None), offset=0), "parray": SimTypePointer(SimTypeBottom(label="SAFEARRAY"), offset=0), "cac": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="CAC", pack=False, align=None), "caub": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="CAUB", pack=False, align=None), "cai": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)}, name="CAI", pack=False, align=None), "caui": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0)}, name="CAUI", pack=False, align=None), "cal": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)}, name="CAL", pack=False, align=None), "caul": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="CAUL", pack=False, align=None), "cah": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="LARGE_INTEGER"), offset=0)}, name="CAH", pack=False, align=None), "cauh": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="ULARGE_INTEGER"), offset=0)}, name="CAUH", pack=False, align=None), "caflt": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeFloat(size=32), offset=0)}, name="CAFLT", pack=False, align=None), "cadbl": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeFloat(size=64), offset=0)}, name="CADBL", pack=False, align=None), "cabool": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0)}, name="CABOOL", pack=False, align=None), "cascode": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)}, name="CASCODE", pack=False, align=None), "cacy": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="CY"), offset=0)}, name="CACY", pack=False, align=None), "cadate": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeFloat(size=64), offset=0)}, name="CADATE", pack=False, align=None), "cafiletime": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), offset=0)}, name="CAFILETIME", pack=False, align=None), "cauuid": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="Guid"), offset=0)}, name="CACLSID", pack=False, align=None), "caclipdata": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="CLIPDATA"), offset=0)}, name="CACLIPDATA", pack=False, align=None), "cabstr": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="CABSTR", pack=False, align=None), "cabstrblob": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="BSTRBLOB"), offset=0)}, name="CABSTRBLOB", pack=False, align=None), "calpstr": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0)}, name="CALPSTR", pack=False, align=None), "calpwstr": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)}, name="CALPWSTR", pack=False, align=None), "capropvar": SimStruct({"cElems": SimTypeInt(signed=False, label="UInt32"), "pElems": SimTypePointer(SimTypeBottom(label="PROPVARIANT"), offset=0)}, name="CAPROPVARIANT", pack=False, align=None), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimTypeBottom(label="SAFEARRAY"), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="PROPVARIANT"), offset=0)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="PROPVARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwId", "hnsOffset", "pvarSegmentOffset"]),
        # 
        'MFCreateAggregateSource': SimTypeFunction([SimTypeBottom(label="IMFCollection"), SimTypePointer(SimTypeBottom(label="IMFMediaSource"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSourceCollection", "ppAggSource"]),
        # 
        'MFCreateCredentialCache': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFNetCredentialCache"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppCache"]),
        # 
        'MFCreateProxyLocator': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IPropertyStore"), SimTypePointer(SimTypeBottom(label="IMFNetProxyLocator"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszProtocol", "pProxyConfig", "ppProxyLocator"]),
        # 
        'MFCreateNetSchemePlugin': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["riid", "ppvHandler"]),
        # 
        'MFCreatePMPServer': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="IMFPMPServer"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dwCreationFlags", "ppPMPServer"]),
        # 
        'MFCreateRemoteDesktopPlugin': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFRemoteDesktopPlugin"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppPlugin"]),
        # 
        'CreateNamedPropertyStore': SimTypeFunction([SimTypePointer(SimTypeBottom(label="INamedPropertyStore"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppStore"]),
        # 
        'MFCreateSampleCopierMFT': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFTransform"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppCopierMFT"]),
        # 
        'MFCreateTranscodeProfile': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFTranscodeProfile"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppTranscodeProfile"]),
        # 
        'MFCreateTranscodeTopology': SimTypeFunction([SimTypeBottom(label="IMFMediaSource"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IMFTranscodeProfile"), SimTypePointer(SimTypeBottom(label="IMFTopology"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrc", "pwszOutputFilePath", "pProfile", "ppTranscodeTopo"]),
        # 
        'MFCreateTranscodeTopologyFromByteStream': SimTypeFunction([SimTypeBottom(label="IMFMediaSource"), SimTypeBottom(label="IMFByteStream"), SimTypeBottom(label="IMFTranscodeProfile"), SimTypePointer(SimTypeBottom(label="IMFTopology"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pSrc", "pOutputStream", "pProfile", "ppTranscodeTopo"]),
        # 
        'MFTranscodeGetAudioOutputAvailableTypes': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFCollection"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["guidSubType", "dwMFTFlags", "pCodecConfig", "ppAvailableTypes"]),
        # 
        'MFCreateTranscodeSinkActivate': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppActivate"]),
        # 
        'MFEnumDeviceSources': SimTypeFunction([SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAttributes", "pppSourceActivate", "pcSourceActivate"]),
        # 
        'MFCreateDeviceSource': SimTypeFunction([SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFMediaSource"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAttributes", "ppSource"]),
        # 
        'MFCreateDeviceSourceActivate': SimTypeFunction([SimTypeBottom(label="IMFAttributes"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAttributes", "ppActivate"]),
        # 
        'MFCreateProtectedEnvironmentAccess': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFProtectedEnvironmentAccess"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppAccess"]),
        # 
        'MFLoadSignedLibrary': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IMFSignedLibrary"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszName", "ppLib"]),
        # 
        'MFGetSystemId': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFSystemId"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppId"]),
        # 
        'MFGetLocalId': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["verifier", "size", "id"]),
        # 
        'MFCreateASFContentInfo': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFASFContentInfo"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppIContentInfo"]),
        # 
        'MFCreateASFIndexer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFASFIndexer"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppIIndexer"]),
        # 
        'MFCreateASFIndexerByteStream': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypeLongLong(signed=False, label="UInt64"), SimTypePointer(SimTypeBottom(label="IMFByteStream"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIContentByteStream", "cbIndexStartOffset", "pIIndexByteStream"]),
        # 
        'MFCreateASFSplitter': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFASFSplitter"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppISplitter"]),
        # 
        'MFCreateASFProfile': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFASFProfile"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppIProfile"]),
        # 
        'MFCreateASFProfileFromPresentationDescriptor': SimTypeFunction([SimTypeBottom(label="IMFPresentationDescriptor"), SimTypePointer(SimTypeBottom(label="IMFASFProfile"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIPD", "ppIProfile"]),
        # 
        'MFCreatePresentationDescriptorFromASFProfile': SimTypeFunction([SimTypeBottom(label="IMFASFProfile"), SimTypePointer(SimTypeBottom(label="IMFPresentationDescriptor"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIProfile", "ppIPD"]),
        # 
        'MFCreateASFMultiplexer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="IMFASFMultiplexer"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppIMultiplexer"]),
        # 
        'MFCreateASFStreamSelector': SimTypeFunction([SimTypeBottom(label="IMFASFProfile"), SimTypePointer(SimTypeBottom(label="IMFASFStreamSelector"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIASFProfile", "ppSelector"]),
        # 
        'MFCreateASFMediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIByteStream", "ppIMediaSink"]),
        # 
        'MFCreateASFMediaSinkActivate': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IMFASFContentInfo"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwszFileName", "pContentInfo", "ppIActivate"]),
        # 
        'MFCreateWMVEncoderActivate': SimTypeFunction([SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IPropertyStore"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMediaType", "pEncodingConfigurationProperties", "ppActivate"]),
        # 
        'MFCreateWMAEncoderActivate': SimTypeFunction([SimTypeBottom(label="IMFMediaType"), SimTypeBottom(label="IPropertyStore"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMediaType", "pEncodingConfigurationProperties", "ppActivate"]),
        # 
        'MFCreateASFStreamingMediaSink': SimTypeFunction([SimTypeBottom(label="IMFByteStream"), SimTypePointer(SimTypeBottom(label="IMFMediaSink"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pIByteStream", "ppIMediaSink"]),
        # 
        'MFCreateASFStreamingMediaSinkActivate': SimTypeFunction([SimTypeBottom(label="IMFActivate"), SimTypeBottom(label="IMFASFContentInfo"), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pByteStreamActivate", "pContentInfo", "ppIActivate"]),
        # 
        'MFCreateVideoRenderer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["riidRenderer", "ppVideoRenderer"]),
        # 
        'MFCreateEncryptedMediaExtensionsStoreActivate': SimTypeFunction([SimTypeBottom(label="IMFPMPHostApp"), SimTypeBottom(label="IStream"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IMFActivate"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pmpHost", "objectStream", "classId", "activate"]),
    }

lib.set_prototypes(prototypes)



from typing import Optional
import logging

import claripy

from ..knowledge_plugins.cfg import CFGModel
from ..analyses.cfg import CFGUtils
from . import Analysis, register_analysis

_l = logging.getLogger(name=__name__)


class CompleteCallingConventionsAnalysis(Analysis):
    """
    Implements full-binary calling convention analysis. During the initial analysis of a binary, you may set
    `recover_variables` to True so that it will perform variable recovery on each function before performing calling
    convention analysis.
    """

    def __init__(self, recover_variables=False, low_priority=False, force=False, cfg: Optional[CFGModel]=None,
                 analyze_callsites: bool=False, skip_signature_matched_functions: bool=False,
                 max_function_blocks: Optional[int]=None, max_function_size: Optional[int]=None):
        """

        :param recover_variables:   Recover variables on each function before performing calling convention analysis.
        :param low_priority:        Run in the background - periodically release GIL.
        :param force:               Perform calling convention analysis on functions even if they have calling
                                    conventions or prototypes already specified (or previously recovered).
        :param cfg:                 The control flow graph model, which will be passed to CallingConventionAnalysis.
        :param analyze_callsites:   Consider artifacts at call sites when performing calling convention analysis.
        :param skip_signature_matched_functions:    Do not perform calling convention analysis on functions that match
                                                    against existing FLIRT signatures.
        :param max_function_blocks: Do not perform calling convention analysis on functions with more than the
                                    specified number of blocks. Setting it to None disables this check.
        :param max_function_size:   Do not perform calling convention analysis on functions whose sizes are more than
                                    `max_function_size`. Setting it to None disables this check.
        """

        self._recover_variables = recover_variables
        self._low_priority = low_priority
        self._force = force
        self._cfg = cfg
        self._analyze_callsites = analyze_callsites
        self._skip_signature_matched_functions = skip_signature_matched_functions
        self._max_function_blocks = max_function_blocks
        self._max_function_size = max_function_size

        self._analyze()

    def _analyze(self):
        """
        Infer calling conventions for all functions in the current project.

        :return:
        """

        # get an ordering of functions based on the call graph
        sorted_funcs = CFGUtils.quasi_topological_sort_nodes(self.kb.functions.callgraph)
        total_funcs = len(sorted_funcs)

        self._update_progress(0)

        for idx, func_addr in enumerate(reversed(sorted_funcs)):
            func = self.kb.functions.get_by_addr(func_addr)

            if (func.calling_convention is None or func.prototype is None) or self._force:
                if func.alignment:
                    # skip all alignments
                    continue

                if self._skip_signature_matched_functions and func.from_signature:
                    # this function matches against a known library function. skip it.
                    continue

                if self._max_function_size is not None:
                    func_size = sum(block.size for block in func.blocks)
                    if func_size > self._max_function_size:
                        _l.info("Skipping variable recovery for %r since its size (%d) is greater than the cutoff "
                                "size (%d).", func, func_size, self._max_function_size)
                        continue

                if self._max_function_blocks is not None:
                    if len(func.block_addrs_set) > self._max_function_blocks:
                        _l.info("Skipping variable recovery for %r since its number of blocks (%d) is greater than the "
                                "cutoff number (%d).", func, len(func.block_addrs_set), self._max_function_blocks)
                        continue

                # if it's a normal function, we attempt to perform variable recovery
                if self._recover_variables and self.function_needs_variable_recovery(func):
                    _l.info("Performing variable recovery on %r...", func)
                    try:
                        _ = self.project.analyses.VariableRecoveryFast(func, kb=self.kb, low_priority=self._low_priority)
                    except claripy.ClaripyError:
                        _l.warning("An claripy exception occurred during variable recovery analysis on function %#x.",
                                   func.addr,
                                   exc_info=True,
                                   )
                        continue

                # determine the calling convention of each function
                cc_analysis = self.project.analyses.CallingConvention(func, cfg=self._cfg, kb=self.kb,
                                                                      analyze_callsites=self._analyze_callsites)
                if cc_analysis.cc is not None:
                    _l.info("Determined calling convention and prototype for %r.", func)
                    func.calling_convention = cc_analysis.cc
                    func.prototype = cc_analysis.prototype
                    func.is_prototype_guessed = True
                else:
                    _l.info("Cannot determine calling convention for %r.", func)

            percentage = (idx + 1) / total_funcs * 100.0
            self._update_progress(percentage)
            if self._low_priority:
                self._release_gil(idx, 10, 0.000001)

    #
    # Static methods
    #

    @staticmethod
    def function_needs_variable_recovery(func):
        """
        Check if running variable recovery on the function is the only way to determine the calling convention of the
        this function.

        We do not need to run variable recovery to determine the calling convention of a function if:
        - The function is a SimProcedure.
        - The function is a PLT stub.
        - The function is a library function and we already know its prototype.

        :param func:    The function object.
        :return:        True if we must run VariableRecovery before we can determine what the calling convention of this
                        function is. False otherwise.
        :rtype:         bool
        """

        if func.is_simprocedure or func.is_plt:
            return False
        # TODO: Check SimLibraries
        return True


register_analysis(CompleteCallingConventionsAnalysis, "CompleteCallingConventions")
